/**
 * Define constants
 */
'use strict';

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
var symbol = _Symbol('associations');

/**
 * Define association decorator creator
 */
var createDecorator = function createDecorator(decoratorName) {
  return function (target) {
    var options = arguments[1] === undefined ? {} : arguments[1];
    return function (Model) {

      // Ensure target exists
      if (typeof target !== 'string' || target === '') {
        throw new Error('Target model name is missing for "' + decoratorName + '" association.');
      }

      var associations = Model.prototype[symbol] = Model.prototype[symbol] || [];

      // Store association in prototype
      associations.push({ method: decoratorName, target: target, options: options });

      return Model;
    };
  };
};

/**
 * Create decorators
 */
var hasOne = createDecorator('hasOne');
var hasMany = createDecorator('hasMany');
var belongsTo = createDecorator('belongsTo');
var belongsToMany = createDecorator('belongsToMany');

/**
 * Export associations decorators
 */
exports.hasOne = hasOne;
exports.hasMany = hasMany;
exports.belongsTo = belongsTo;
exports.belongsToMany = belongsToMany;
exports.symbol = symbol;