/**
 * Import dependencies
 */
'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _defineProperty = require('babel-runtime/helpers/define-property')['default'];

var _toConsumableArray = require('babel-runtime/helpers/to-consumable-array')['default'];

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _sequelizeLibDataTypes = require('sequelize/lib/data-types');

var _sequelizeLibDataTypes2 = _interopRequireDefault(_sequelizeLibDataTypes);

/**
 * Define constants
 */
var symbol = _Symbol('attributes');

var typeKeys = [].concat(_toConsumableArray(_Object$keys(_sequelizeLibDataTypes2['default'])), _toConsumableArray(_Object$keys(_sequelizeLibDataTypes2['default']).map(function (type) {
  return type.toLowerCase();
})));

var whitelistKeys = ['type', 'allowNull', 'defaultValue', 'unique', 'primaryKey', 'field', 'autoIncrement', 'comment', 'references', 'onUpdate', 'onDelete', 'get', 'set', 'validate'].concat(_toConsumableArray(typeKeys), ['joi', 'faker']);

var defaultValues = {
  allowNull: true,
  unique: true,
  primaryKey: true,
  autoIncrement: true
};

/**
 * Convert options into chainable decorator
 */
var attribute = whitelistKeys.reduce(function (map, key) {
  return _extends({}, map, _defineProperty({}, key, function () {
    // Get options
    var properties = _Object$assign({}, this);
    var options = properties[symbol] = properties[symbol] || {};

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    // Set data type options
    if (typeKeys.indexOf(key) !== -1) {
      var _DataTypes$key$toUpperCase;

      options.type = (_DataTypes$key$toUpperCase = _sequelizeLibDataTypes2['default'][key.toUpperCase()]).bind.apply(_DataTypes$key$toUpperCase, [null].concat(args));
    }
    // Set other options
    else {
      options[key] = typeof args[0] !== 'undefined' ? args[0] : defaultValues[key];
    }

    // Create decorator
    var decorator = function decorator(prototype, attributeName, descriptor) {
      // Get property initializer
      var initializer = typeof descriptor.initializer === 'function' ? descriptor.initializer() : descriptor.initializer;

      // Set default value from initializer
      if (initializer !== null) {
        options.defaultValue = initializer;
        if (options.joi) {
          options.joi = options.joi['default'](initializer, attributeName);
        }
      }

      // Store attribute options in prototype
      prototype[symbol] = _extends({}, prototype[symbol] || {}, _defineProperty({}, attributeName, options));

      // Return property descriptor
      return descriptor;
    };

    // Return chainable decorator
    return _Object$assign(decorator, properties);
  }));
}, {});

/**
 * Export attribute decorator
 */
exports['default'] = attribute;
exports.attribute = attribute;
exports.symbol = symbol;