/**
 * Import dependencies
 */
'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _defineProperty = require('babel-runtime/helpers/define-property')['default'];

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _Object$defineProperties = require('babel-runtime/core-js/object/define-properties')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _arrayifyCompact = require('arrayify-compact');

var _arrayifyCompact2 = _interopRequireDefault(_arrayifyCompact);

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _utilsEnsurePropsEnumerability = require('../utils/ensurePropsEnumerability');

var _utilsEnsurePropsEnumerability2 = _interopRequireDefault(_utilsEnsurePropsEnumerability);

var _optionsGetBaseOptions = require('../options/getBaseOptions');

var _optionsGetBaseOptions2 = _interopRequireDefault(_optionsGetBaseOptions);

var _optionsGetJoiOptions = require('../options/getJoiOptions');

var _optionsGetJoiOptions2 = _interopRequireDefault(_optionsGetJoiOptions);

var _optionsGetFakerOptions = require('../options/getFakerOptions');

var _optionsGetFakerOptions2 = _interopRequireDefault(_optionsGetFakerOptions);

var _attribute = require('./attribute');

var _associations = require('./associations');

/**
 * Define constants
 */
var optionsSymbol = _Symbol('options');

/**
 * Define model decorator
 */
var model = function model(name) {
  var options = arguments[1] === undefined ? {} : arguments[1];
  return function (Class) {
    // Ensure name is defined
    if (typeof name !== 'string' || name === '') {
      throw new Error('Model name is missing.');
    }

    // Ensure static/prototype properties are enumerable
    (0, _utilsEnsurePropsEnumerability2['default'])(Class);

    // Ensure hook options are arrays
    if (options.hooks) {
      options.hooks = _Object$keys(options.hooks).reduce(function (map, key) {
        return _extends({}, map, _defineProperty({}, key, (0, _arrayifyCompact2['default'])(options.hooks[key])));
      }, {});
    }

    // Store associations in options
    options.associationsQueue = Class.prototype[_associations.symbol] || [];

    // Create options
    var opts = (0, _deepmerge2['default'])(options, (0, _optionsGetBaseOptions2['default'])(Class));
    opts = (0, _deepmerge2['default'])(opts, (0, _optionsGetJoiOptions2['default'])());
    opts = (0, _deepmerge2['default'])(opts, (0, _optionsGetFakerOptions2['default'])());
    Class.prototype[optionsSymbol] = opts;

    // Create argument list
    var args = [name, Class.prototype[_attribute.symbol], Class.prototype[optionsSymbol]];

    // Add access to internals
    _Object$defineProperties(args, {
      name: {
        get: function get() {
          return name;
        }
      },
      attributes: {
        get: function get() {
          return Class.prototype[_attribute.symbol];
        }
      },
      options: {
        get: function get() {
          return Class.prototype[optionsSymbol];
        }
      }
    });

    // Return list of argument for "sequelize.define" function
    return args;
  };
};

/**
 * Export model decorator
 */
exports['default'] = model;
module.exports = exports['default'];