/**
 * Import dependencies
 */
'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _toConsumableArray = require('babel-runtime/helpers/to-consumable-array')['default'];

var _bind = require('babel-runtime/helpers/bind')['default'];

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _sequelize = require('sequelize');

var _sequelize2 = _interopRequireDefault(_sequelize);

/**
 * Define constants
 */
var symbol = _Symbol('queue');

/**
 * Define associate hook
 */
var associate = function associate(Model) {
  // Get models
  var models = Model.sequelize.models;

  // Get queue
  var queue = Model.sequelize[symbol] = Model.sequelize[symbol] || [];

  // Get associations
  var associations = Model.options.associationsQueue;

  // Handle queue
  var it = queue.length;
  while (it--) {
    var _queue$it = queue[it];
    var method = _queue$it.method;
    var source = _queue$it.source;
    var target = _queue$it.target;
    var options = _queue$it.options;

    if (Model.name === target) {
      queue.splice(it, 1);
      models[source][method](models[target], options);
    }
  }

  // Handle associations
  it = associations.length;
  while (it--) {
    var _associations$it = associations[it];
    var method = _associations$it.method;
    var _associations$it$source = _associations$it.source;
    var source = _associations$it$source === undefined ? Model.name : _associations$it$source;
    var target = _associations$it.target;
    var options = _associations$it.options;

    if (models[target]) {
      associations.splice(it, 1);
      models[source][method](models[target], options);
    } else {
      queue.push({ method: method, source: source, target: target, options: options });
    }
  }
};

/**
 * Define Sequelize wrapper
 */

var SequelizeWrapper = function SequelizeWrapper() {
  _classCallCheck(this, SequelizeWrapper);

  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  // Create Sequelize instance
  var sequelize = new (_bind.apply(_sequelize2['default'], [null].concat(_toConsumableArray(args))))();

  // Add hook to create associations
  sequelize.addHook('afterDefine', associate);

  // Return sequelize instance
  return sequelize;
};

/**
 * Export Sequelize wrapper and hook
 */
exports['default'] = SequelizeWrapper;
exports.SequelizeWrapper = SequelizeWrapper;
exports.associate = associate;